/*
 * Copyright © 2011 Keith Packard <keithp@keithp.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 */

#include "ao.h"

#define COMPANION_SELECT()	ao_spi_get_bit(COMPANION_CS)
#define COMPANION_DESELECT()	ao_spi_put_bit(COMPANION_CS)

static __xdata struct ao_companion_command	ao_companion_command;
__xdata struct ao_companion_setup		ao_companion_setup;

__xdata uint16_t	ao_companion_data[AO_COMPANION_MAX_CHANNELS];
__pdata uint8_t		ao_companion_running;
__xdata uint8_t		ao_companion_mutex;

static void
ao_companion_send_command(uint8_t command)
{
	ao_companion_command.command = command;
	ao_companion_command.flight_state = ao_flight_state;
	ao_companion_command.tick = ao_time();
	ao_spi_send(&ao_companion_command, sizeof (ao_companion_command));
}

static uint8_t
ao_companion_get_setup(void)
{
	COMPANION_SELECT();
	ao_companion_send_command(AO_COMPANION_SETUP);
	ao_spi_recv(&ao_companion_setup, sizeof (ao_companion_setup));
	COMPANION_DESELECT();
	if (ao_companion_setup.board_id != ~ao_companion_setup.board_id)
		return 0;
	return 1;
}

static void
ao_companion_get_data(void)
{
	COMPANION_SELECT();
	ao_companion_send_command(AO_COMPANION_FETCH);
	ao_mutex_get(&ao_companion_mutex);
	ao_spi_recv(&ao_companion_data, ao_companion_setup.channels * 2);
	ao_mutex_put(&ao_companion_mutex);
	COMPANION_DESELECT();
}

void
ao_companion(void)
{
	if (!ao_companion_get_setup())
		ao_exit();
	ao_companion_running = 1;
	for (;;) {
		ao_delay(ao_companion_setup.update_period);
		ao_companion_get_data();
	}
}

void
ao_companion_status(void) __reentrant
{
}

__code struct ao_cmds ao_companion_cmds[] = {
	{ ao_companion_status,	"L\0Companion link status" },
	{ 0, NULL },
};

static __xdata struct ao_task ao_companion_task;

void
ao_companion_init(void)
{
	COMPANION_CS_PORT |= COMPANION_CS_MASK;	/* raise all CS pins */
	COMPANION_CS_DIR |= COMPANION_CS_MASK;	/* set CS pins as outputs */
	COMPANION_CS_SEL &= ~COMPANION_CS_MASK;	/* set CS pins as GPIO */

	ao_cmd_register(&ao_companion_cmds[0]);

	ao_add_task(&ao_companion_task, ao_companion, "companion");
}
